module D95
  class MoveTool
    def initialize(imported_entities)
      @imported_entities = imported_entities
      @ip = Sketchup::InputPoint.new
      @transformation = nil
    end

    def onMouseMove(flags, x, y, view)
      @ip.pick(view, x, y)
      view.invalidate

      # Calculate the translation vector
      if @transformation
        view.model.entities.transform_entities(@transformation.inverse, @imported_entities)
      end

      translation = Geom::Transformation.translation(@ip.position - @bounding_box.center)
      @transformation = translation
      view.model.entities.transform_entities(@transformation, @imported_entities)
    end

    def draw(view)
      @ip.draw(view) if @ip.valid?
    end

    def onLButtonDown(flags, x, y, view)
      # Finalize the position
      Sketchup.active_model.select_tool(nil) # Exit the tool
    end
  end

  def self.pasteFromCAD
    tempPath = Sketchup.temp_dir
    dwgList = Dir.glob(File.join(tempPath, "*.DWG")).max_by { |f| File.mtime(f) }
    model = Sketchup.active_model
    entities = model.entities
  
    # Record existing entities before import
    existing_entities = entities.to_a
  
    # Import the DWG file
    model.import dwgList
  
    # Get the bounding box of the imported entities
    imported_entities = entities.to_a - existing_entities
    bounding_box = Geom::BoundingBox.new
    imported_entities.each { |entity| bounding_box.add(entity.bounds) }
  
    # Activate the tool to move the imported entities
    tool = MoveTool.new(imported_entities)
    tool.instance_variable_set(:@bounding_box, bounding_box)
    model.select_tool(tool)
  end

  menu = UI.menu('Extensions')
  menu.add_item("从 AutoCAD 粘贴") { D95.pasteFromCAD }
end

